<?php
namespace Internetgalerie\Flipbook\Utility;

use TYPO3\CMS\Core\Resource\FileRepository;
use TYPO3\CMS\Core\Resource\StorageRepository;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Core\Database\ConnectionPool;
use TYPO3\CMS\Extbase\Persistence\Generic\PersistenceManager;
use TYPO3\CMS\Frontend\ContentObject\ContentObjectRenderer;
//TYPO3\CMS\FrontendEditing\Xclass\ContentObject\ContentObjectRenderer;

class FlipbookUtility {
    protected $outputBasepath = '/fileadmin/flipbook/';
    //protected $outputYmlFile = 'flipbook.xml';
    protected $outputFlipbookDir = null;
    protected $outputImageFile = 'flipbook-%03d.jpg';
    protected $outputPathAbs = null;
    protected $outputPageWidth = '2100';
    protected $outputPageHeight = '3000';
    protected $zoomImageWidth = '3000';
    protected $zoomImageHeight = '3000';
    protected $density = '300';
    protected $alpha = 'remove';
    // 'off'; (scheint schlechter zu sein)
    protected $quality = '90';
    protected $background = 'white';
    protected $id = '';
    protected $pdfFile = '';
    protected $touchPath = '';
    protected $docRoot = '';


    public function init(ContentObjectRenderer $contentObject)
    {
        $this->docRoot = getenv('DOCUMENT_ROOT');
        $this->outputFlipbookDir = $contentObject->data['uid'];
        /*
          if ($this->contentObj->data['uid']) {
          $this->outputFlipbookDir = $this->contentObj->data['uid'];
          } else {
          $this->outputFlipbookDir = basename($pdfFile);
          }
        */
        $outputPathRel = $this->pathCat($this->outputBasepath, $this->outputFlipbookDir);
        $this->outputPathAbs = $this->pathCat($this->docRoot, $outputPathRel);
        $this->touchPath = $this->pathCat($this->outputPathAbs, 'time.txt');

    }
    public function getPageImagesByContentObject(ContentObjectRenderer $contentObject, ?string $pageRange = null)
    {
        $this->init($contentObject);

        $uid = (int)$contentObject->data['uid'];
        $pageImages = $this->getPageImageByUid($uid);

        //$this->settings['convertNew']
        if ($this->mustUpdate((int)$contentObject->data['tstamp']) || count($pageImages)==0) {
            $pdfFiles = $this->getPdfFilesByContentObject($contentObject);
   
            $convertPdf = false;
            foreach ($pdfFiles as $key => $file) {
                $identifier = $file->getIdentifier();
      
                // if pdf render IMGS first and ADD/MERGE to $pageImages Array
                if (pathinfo($identifier, PATHINFO_EXTENSION) == 'pdf') {
                    $convertPdf = $this->convertPdf($file->getPublicUrl(), $contentObject, $pageRange);
                    //$pdf[] = $file;
                }
            }
            $pageImages = $this->getPageImageByUid($uid);
        }
        return $pageImages;
    }
    public function getPdfFilesByContentObject(ContentObjectRenderer $contentObject): array
    {
        $fileRepository = GeneralUtility::makeInstance(FileRepository::class);
        // Get PDF
        $fileObjects = $fileRepository->findByRelation('tt_content', 'files', $contentObject->data['uid']);
        return $fileObjects;
    }

    public function getPageImageByUid(int $uid)
    {
        $fileRepository = GeneralUtility::makeInstance(FileRepository::class);

        // sind umgewandelte Bilder da
        $fileObjects = $fileRepository->findByRelation('tt_content', 'pageimages', $uid);
        //if(count($fileObjects)>0 || $noPdf)
        return $fileObjects;
    }

    // Funktion zum zusammenf�gen von Pfade ( z.B. '/dir/'.'/path2/etc'  oder '/dir'.'path2/etc' )
    public function pathCat(string $base_path, string $rel_path): string
    {
        return rtrim($base_path, '/') . '/' . ltrim($rel_path, '/');
    }

    public function convertPdf(string $pdfFile, ContentObjectRenderer $contentObject, ?string $pageRange = null): bool
    {
        $large_generate = 1;

        if (!$this->outputPageHeight) {
            $page_height = $this->outputPageWidth * 2;
        } else {
            $page_height = $this->outputPageHeight;
        }

        if (!$this->zoomImageHeight) {
            $zoomImageHeight = $this->zoomImageWidth * 2;
        } else {
            $zoomImageHeight = $this->zoomImageHeight;
        }

        $fullImagesFilePattern = $this->pathCat($this->outputPathAbs, $this->outputImageFile);
        $fullPdfPath = $this->pathCat($this->docRoot, $pdfFile);
        
        // Konstanten
        //$cmd = '/usr/bin/convert -verbose -colorspace sRGB -density ' . $this->density . ' -quality ' . $this->quality . ' -background ' . $this->background . ' -alpha ' . $this->alpha . ' -geometry ';
        $cmd = 'gm convert -verbose +adjoin -colorspace RGB -density ' . $this->density . ' -quality ' . $this->quality . ' -background ' . $this->background . ' -geometry ';
        //Befehl ausfuehren zum Umwandeln des PDF
        $output_size = $this->outputPageWidth . 'x' . $page_height;
        // Ist PDF bereits umgewandelt
        if (is_dir($this->outputPathAbs) /*|| $this->settings['convertNew'] == 1*/) {
            if ($dh = opendir($this->outputPathAbs)) {
                while (($file = readdir($dh)) !== false) {
                    $filePath = $this->pathCat($this->outputPathAbs, $file);
                    if (is_file($filePath)) {
                        unlink($filePath);
                        // remove sys_file_reference * sys_file based on object UID and filepath
                        if (is_numeric($this->outputFlipbookDir) && pathinfo($filePath, PATHINFO_EXTENSION) != '.txt') {
                            $this->removeOldFileFromDB($filePath);
                        }
                    }
                }
                closedir($dh);
            }
        }
        if (!is_dir($this->outputPathAbs)) {
            mkdir($this->outputPathAbs, 0755, 1);
        }
        file_put_contents($this->touchPath, $contentObject->data['tstamp']);
        if ($pageRange !== null && $pageRange !== '' && preg_match("/^[-,0-9]+$/i", $pageRange)) {
            $pageRangeCmd = '[' . $pageRange . ']';
        } else {
            $pageRangeCmd = '';
        }

        //die($cmd . $output_size . ' ' . $fullPdfPath . ' ' . $fullImagesFilePattern);
        $fp = popen($cmd . $output_size . ' ' . $fullPdfPath . $pageRangeCmd . ' ' . $fullImagesFilePattern, 'r');
        pclose($fp);
        $storageRepository = GeneralUtility::makeInstance(StorageRepository::class);
        $storage = $storageRepository->findByUid(1);

        $saveFolder = $storage->getFolder('flipbook/' . $this->outputFlipbookDir);
        //$files = scandir($this->outputPathAbs);
        // glob ignores txt and doens't write it in DB
        $pageImages = glob($this->outputPathAbs.'/flipbook-*.jpg');
        $persistenceManager = GeneralUtility::makeInstance(PersistenceManager::class);
            
        //echo('count='. count($pageImages));
        foreach ($pageImages as $key => $pageImage) {
            $sorting=1+intval(substr($pageImage, strrpos($pageImage, '-')+1));  // File flipbook-027.jpg -> Zahl hinter hinterstem -
                  
            $pageImageObject = $storage->addFile('fileadmin' . $saveFolder->getIdentifier() . '/' . basename($pageImage), $saveFolder, basename($pageImage), 'replace');
            $persistenceManager->persistAll();
            $falReference = [
                'pid' => $contentObject->data['pid'],
                'uid_local' => $pageImageObject->getUid(),
                'uid_foreign' => $contentObject->data['uid'],
                'tablenames' => 'tt_content',
                'fieldname' => 'pageimages',
                'sorting_foreign' => $sorting
            ];
            $queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)->getQueryBuilderForTable('sys_file_reference');
            if (!$this->getReferenz($pageImageObject->getUid())) {
                $queryBuilder->insert('sys_file_reference')->values($falReference)->executeStatement();
                //$queryBuilder->insert('sys_file_reference')->values($falReference)->executeStatement();
            } else {
                $queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)->getQueryBuilderForTable('sys_file_reference');
                $queryBuilder->update('sys_file_reference')
                             ->where(
                                 $queryBuilder->expr()->eq('uid_local', $pageImageObject->getUid())
                             );
                foreach ($falReference as $key => $field) {
                    $queryBuilder->set($key, $field);
                }
                $queryBuilder->executeStatement();
                //$queryBuilder->executeStatement();
            }
            //}
        }
        return true;
    }

    public function getReferenz(int $uid)
    {
        $queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)->getQueryBuilderForTable('sys_file_reference');
        $result = $queryBuilder->select('uid')
                               ->from('sys_file_reference')
                               ->where(
                                   $queryBuilder->expr()->eq(
                                       'uid_local',
                                       $uid
                                   )
                               )
                               ->executeQuery()->fetchAllAssociative();
        //->executeQuery()->fetchAllAssociative();
        return $result;
    }

    public function deleteOldEntries(int $uid)
    {
        $queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)->getQueryBuilderForTable('sys_file_reference');
        $queryBuilder->delete('sys_file_reference')
                     ->where(
                         $queryBuilder->expr()->eq(
                             'uid_local',
                             $uid
                         )
                     )->executeStatement();
        //)->executeStatement();
    }
    public function mustUpdate(int $contentObjectTimeStamp): bool
    {
        if (!file_exists($this->touchPath)) {
            return true;
        }
        $touchTstamp = (int)file_get_contents($this->touchPath);
        return $touchTstamp !== $contentObjectTimeStamp;
    }

    public function removeOldFileFromDB(string $filePath)
    {
        // removes images from sys_file
        $queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)->getQueryBuilderForTable('sys_file');
        $queryBuilder->delete('sys_file')
                     ->where(
                         $queryBuilder->expr()->eq(
                             'identifier',
                             $queryBuilder->createNamedParameter('/flipbook/' . $this->outputFlipbookDir.'/'.basename($filePath))
                         )
                     )->executeStatement();
        //)->executeStatement();
        //removes sys_file_references based on filename pageimages and UID
        $queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)->getQueryBuilderForTable('sys_file_reference');
        $queryBuilder->delete('sys_file_reference')
                     ->where(
                         $queryBuilder->expr()->eq(
                             'uid_foreign',
                             $this->outputFlipbookDir
                         ),
                         $queryBuilder->expr()->eq(
                             'fieldname',
                             $queryBuilder->createNamedParameter('pageimages')
                         )
                     )->executeStatement();
        //)->executeStatement();
    }

}
